import { IDictionary } from "@globalpayments/js/types/internal/lib/util";
import { IframeField } from "@globalpayments/js/types/ui";
import UIForm from "@globalpayments/js/types/ui/form";
import { SecurePaymentElement } from "./secure-payment-element";
/**
 * @slot - When the default slot is empty, the drop-in form is rendered.
 *
 * @fires ready - Triggered when all form fields have been rendered successfully.
 * @fires submit - Triggered when submit button is clicked.
 * @fires token-success - Triggered when a successful token response is received from the configured service.
 * @fires token-error - Triggered when an errored token response is received from the configured service.
 * @fires error - Triggered when a general error occurs.
 */
export declare class SecurePaymentForm extends SecurePaymentElement {
    protected formType: string | undefined;
    protected shadow: ShadowRoot;
    protected form: any;
    protected dropin: boolean;
    protected elements: {
        [key: string]: SecurePaymentElement;
    };
    /**
     * Element constructor.
     *
     * Creates a shadow root for the element, and configures the Global Payments
     * JavaScript library based on set attributes.
     */
    constructor();
    /**
     * Configures the Global Payments JavaScript library.
     */
    configure(): Promise<void>;
    /**
     * Callback for when the element is attached to the DOM.
     *
     * Renders the form.
     */
    connectedCallback(): void;
    /**
     * Callback for when the element is attached to the DOM.
     *
     * Disposes the form.
     */
    disconnectedCallback(): void;
    /**
     * @inheritdoc
     *
     * @returns An empty object if no styles have been set.
     */
    getStyles(): {};
    /**
     * @inheritdoc
     */
    getTargetEvents(): string[];
    /**
     * Renders the secure payment form with hosted fields.
     */
    render(): Promise<void>;
    setupEventListeners(source: UIForm | IframeField): void;
    /**
     * Finds all implemented secure payment elements, and creates promises
     * to wait for them to be defined using `customElements.whenDefined`.
     *
     * @returns The promises for custom element definitions
     */
    protected allNeededElementsDefined(): Promise<void[]>;
    /**
     * Extracts field-level options for creating the form with the
     * Global Payments JavaScript library.
     *
     * @param isDropin
     *
     * @returns The form options
     */
    protected extractOptions(isDropin?: boolean): {
        fields?: undefined;
        styles?: undefined;
    } | {
        fields: IDictionary;
        styles: {};
    };
    /**
     * Grabs the first form child of a given type.
     *
     * To simplify things, integrators implement custom field elements
     * (e.g. `secure-card-number-field`) as children of `secure-payment-form`.
     * This removes the need for custom `id` or `class` attribute values, but
     * the form needs to ensure that only one element of each type is used
     * when configuring the Global Payment JavaScript library.
     *
     * @param type
     *
     * @returns `Element` when a match is found; `null` when no matches
     */
    protected findFirstChild(type: string): any;
    /**
     * Gets the `node`"s `nodeName` property in lower case to be used as the
     * hosted fields target / query selector.
     *
     * @param node
     *
     * @returns Name as a string; Empty string if node is `null`/`undefined`
     */
    protected nodeAsTarget(node: Node): string;
}
declare global {
    interface HTMLElementTagNameMap {
        "secure-payment-form": SecurePaymentForm;
    }
}
