let field;
class SecurePaymentElement extends HTMLElement {
    constructor() {
        super(...arguments);
        // tslint:disable-next-line:no-empty
        this.on = () => { };
    }
    /**
     * Sets the underlying hosted field.
     *
     * Exposes functionality from the hosted field object to the custom
     * element's JavaScript interface.
     *
     * @param value
     */
    set field(value) {
        field = value;
        this.on = field.on.bind(this);
    }
    /**
     * Gets the underlying hosted field.
     */
    get field() {
        return field;
    }
    /**
     * Gets the custom element's pre-defined tag name.
     */
    get tagName() {
        return "";
    }
    /**
     * Gets the hosted field type.
     *
     * Current options are:
     *
     * - `card-number`
     * - `card-expiration`
     * - `card-cvv`
     * - `submit`
     */
    get type() {
        return "";
    }
    /**
     * @returns List of event types supported on the element.
     */
    getTargetEvents() {
        return ["ready"];
    }
    /**
     * Gets the element's attributes as an object, replacing kebab cased attribute
     * names with their camel cased equivalents.
     */
    getAttributes() {
        const attributes = {};
        // tslint:disable-next-line:prefer-for-of
        for (let i = 0; i < this.attributes.length; i++) {
            const att = this.attributes[i];
            attributes[this.kebabCaseToCamelCase(att.name)] = att.value;
        }
        return attributes;
    }
    /**
     * Gets hosted fields styles if they have been set on the custom element.
     *
     * @returns An empty object if no styles have been set.
     */
    getStyles() {
        let result = {};
        if (this.hasStyles()) {
            try {
                result = this.scopeInputSelectors(JSON.parse(this.getAttribute(SecurePaymentElement.ATTRIBUTE_NAME_STYLES) || ""));
            }
            catch (e) { /** */ }
        }
        return result;
    }
    /**
     * Determines if hosted fields styles have been set on the custom element.
     */
    hasStyles() {
        return this.hasAttribute(SecurePaymentElement.ATTRIBUTE_NAME_STYLES);
    }
    /**
     * Sets up event listeners to forward events from the Global Payments
     * JavaScript library to be dispatched as `CustomEvent`s from this
     * element.
     *
     * @param source
     */
    setupEventListeners(source) {
        if (!source) {
            return;
        }
        const dispatch = (target, event) => {
            return (detail) => target.dispatchEvent(new CustomEvent(event, { detail }));
        };
        this.getTargetEvents().forEach((event) => source.on(event, dispatch(this, event)));
    }
    /**
     * Converts kebab case (`kebab-case`) to camel case (`camelCase`).
     *
     * @param name
     */
    kebabCaseToCamelCase(name) {
        const parts = name.split("-");
        if (parts.length === 1) {
            return parts[0];
        }
        for (let i = 1; i < parts.length; i++) {
            const part = parts[i];
            parts[i] = part.substring(0, 1).toUpperCase() + part.substring(1);
        }
        return parts.join("");
    }
    /**
     * Replaces generic CSS selectors (e.g. `input`) with alternatives for the specific
     * input field.
     *
     * When configuring all hosted fields, all CSS is compiled together and sent to each
     * field that is constructed. Scoping CSS selectors prevents unintended side effects.
     *
     * @param styles
     */
    scopeInputSelectors(styles) {
        /** @type {IDictionary} */
        const result = {};
        Object.keys(styles).forEach((key) => {
            let value = styles[key];
            if (typeof value !== "string") {
                value = this.scopeInputSelectors(value);
            }
            if (key.indexOf("input") !== -1 || key.indexOf("#secure-payment-field") !== -1) {
                key = `.${this.type}`;
            }
            result[key] = value;
        });
        return result;
    }
    /**
     * Upgrades a property when the custom element is defined.
     *
     * @param prop Property name
     */
    upgradeProperty(prop) {
        if (this.hasOwnProperty(prop)) {
            const self = this;
            const value = self[prop];
            delete self[prop];
            self[prop] = value;
        }
    }
}
/**
 * RegEx for finding all valid secure payment element names
 */
SecurePaymentElement.FIND_ALL_REGEX = /secure-([\w\-]+)-(field|button)/ig;
/**
 * Attribute name for custom iframe styles
 */
SecurePaymentElement.ATTRIBUTE_NAME_STYLES = "gp-style";

class SecureCardNumberField extends SecurePaymentElement {
    /**
     * @inheritdoc
     */
    get tagName() {
        return SecureCardNumberField.TAG_NAME;
    }
    /**
     * @inheritdoc
     */
    get type() {
        return "card-number";
    }
}
SecureCardNumberField.TAG_NAME = "secure-card-number-field";
customElements.define(SecureCardNumberField.TAG_NAME, SecureCardNumberField);

class SecureCardExpirationField extends SecurePaymentElement {
    /**
     * @inheritdoc
     */
    get tagName() {
        return SecureCardExpirationField.TAG_NAME;
    }
    /**
     * @inheritdoc
     */
    get type() {
        return "card-expiration";
    }
}
SecureCardExpirationField.TAG_NAME = "secure-card-expiration-field";
customElements.define(SecureCardExpirationField.TAG_NAME, SecureCardExpirationField);

class SecureCardCvvField extends SecurePaymentElement {
    /**
     * @inheritdoc
     */
    get tagName() {
        return SecureCardCvvField.TAG_NAME;
    }
    /**
     * @inheritdoc
     */
    get type() {
        return "card-cvv";
    }
}
SecureCardCvvField.TAG_NAME = "secure-card-cvv-field";
customElements.define(SecureCardCvvField.TAG_NAME, SecureCardCvvField);

class SecureSubmitButton extends SecurePaymentElement {
    /**
     * @inheritdoc
     */
    get tagName() {
        return SecureSubmitButton.TAG_NAME;
    }
    /**
     * @inheritdoc
     */
    get type() {
        return "submit";
    }
    /**
     * @inheritdoc
     */
    getTargetEvents() {
        return Array.of(...super.getTargetEvents(), ...["click"]);
    }
}
SecureSubmitButton.TAG_NAME = "secure-submit-button";
customElements.define(SecureSubmitButton.TAG_NAME, SecureSubmitButton);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

/// <reference types="@globalpayments/js" />
/**
 * Prevent multiple copies of the Global Payments JavaScript
 * library from being loaded by only using a single Promise.
 */
let libraryLoaded;
/**
 * Loads the Global Payments JavaScript library asynchronously, using a
 * resolved `Promise` to signal that the library has loaded.
 *
 * @param url
 *
 * @returns The resolved library
 */
function loadLibrary(url) {
    if (libraryLoaded) {
        return libraryLoaded;
    }
    if (window.GlobalPayments) {
        return libraryLoaded = Promise.resolve(window.GlobalPayments);
    }
    if (!url) {
        url = "https://js.globalpay.com/v1/globalpayments.js";
    }
    return libraryLoaded = new Promise((resolve) => {
        try {
            const script = document.createElement("script");
            script.defer = true;
            script.src = url || "";
            script.onload = () => {
                resolve(window.GlobalPayments);
            };
            document.body.appendChild(script);
        }
        catch (e) { /** */ }
    });
}

/// <reference types="@globalpayments/js" />
const fieldTypeTagNameMap = {
    "card-number": SecureCardNumberField.TAG_NAME,
    "card-expiration": SecureCardExpirationField.TAG_NAME,
    "card-cvv": SecureCardCvvField.TAG_NAME,
    "submit": SecureSubmitButton.TAG_NAME,
};
const fieldTypes = Object.keys(fieldTypeTagNameMap);
/**
 * @slot - When the default slot is empty, the drop-in form is rendered.
 *
 * @fires ready - Triggered when all form fields have been rendered successfully.
 * @fires submit - Triggered when submit button is clicked.
 * @fires token-success - Triggered when a successful token response is received from the configured service.
 * @fires token-error - Triggered when an errored token response is received from the configured service.
 * @fires error - Triggered when a general error occurs.
 */
class SecurePaymentForm extends SecurePaymentElement {
    /**
     * Element constructor.
     *
     * Creates a shadow root for the element, and configures the Global Payments
     * JavaScript library based on set attributes.
     */
    constructor() {
        super();
        this.shadow = this.attachShadow({ mode: "open" });
        this.dropin = false;
        this.elements = {};
        this.shadow.appendChild(document.createElement("slot"));
        this.configure();
    }
    /**
     * Configures the Global Payments JavaScript library.
     */
    configure() {
        return __awaiter(this, void 0, void 0, function* () {
            yield loadLibrary();
            const options = this.getAttributes();
            if (options.formType !== undefined) {
                this.formType = options.formType;
                this.removeAttribute("form-type");
            }
            window.GlobalPayments.configure(options);
        });
    }
    /**
     * Callback for when the element is attached to the DOM.
     *
     * Renders the form.
     */
    connectedCallback() {
        if (this.form) {
            return;
        }
        this.render();
    }
    /**
     * Callback for when the element is attached to the DOM.
     *
     * Disposes the form.
     */
    disconnectedCallback() {
        if (!this.form) {
            return;
        }
        this.form.dispose();
    }
    /**
     * @inheritdoc
     *
     * @returns An empty object if no styles have been set.
     */
    getStyles() {
        let result = {};
        if (this.hasStyles()) {
            try {
                result = JSON.parse(this.getAttribute(SecurePaymentElement.ATTRIBUTE_NAME_STYLES) || "");
            }
            catch (e) { /** */ }
        }
        return result;
    }
    /**
     * @inheritdoc
     */
    getTargetEvents() {
        return Array.of(...super.getTargetEvents(), ...["token-success", "token-error", "error"]);
    }
    /**
     * Renders the secure payment form with hosted fields.
     */
    render() {
        return __awaiter(this, void 0, void 0, function* () {
            yield loadLibrary();
            if (this.dropin === true) {
                // render drop-in form
                return;
            }
            // render the hosted fields
            try {
                // allow elements to be loaded/defined asynchronously
                yield this.allNeededElementsDefined();
                this.form = window.GlobalPayments.ui.form(this.extractOptions());
            }
            catch (e) {
                this.form = window.GlobalPayments.creditCard.form(this, {
                    // @ts-ignore
                    style: this.getAttribute("theme") || undefined,
                });
            }
            // get each hosted field
            for (const type in this.form.frames) {
                if (!this.form.frames.hasOwnProperty(type)) {
                    continue;
                }
                const frame = this.form.frames[type];
                const element = this.elements[type];
                if (!element) {
                    continue;
                }
                element.field = frame;
                element.setupEventListeners(element.field);
            }
            this.setupEventListeners(this.form);
        });
    }
    setupEventListeners(source) {
        super.setupEventListeners(source);
        window.GlobalPayments.on("error", (detail) => this.dispatchEvent(new CustomEvent("error", { detail })));
        if (typeof source.ready !== "undefined") {
            source.ready((detail) => this.dispatchEvent(new CustomEvent("ready", { detail })));
            // custom form events
            source.on("submit", "click", (detail) => this.dispatchEvent(new CustomEvent("submit", { detail })));
        }
    }
    /**
     * Finds all implemented secure payment elements, and creates promises
     * to wait for them to be defined using `customElements.whenDefined`.
     *
     * @returns The promises for custom element definitions
     */
    allNeededElementsDefined() {
        const childTags = [];
        const matches = this.innerHTML.match(SecurePaymentElement.FIND_ALL_REGEX);
        if (!matches) {
            return Promise.reject();
        }
        matches.forEach((tag) => {
            if (childTags.indexOf(tag) === -1) {
                childTags.push(tag);
            }
        });
        return Promise.all(childTags.map((tag) => customElements.whenDefined(tag)));
    }
    /**
     * Extracts field-level options for creating the form with the
     * Global Payments JavaScript library.
     *
     * @param isDropin
     *
     * @returns The form options
     */
    extractOptions(isDropin = false) {
        const fields = {};
        let styles = this.getStyles();
        if (isDropin) {
            return {};
        }
        // gather field/style configurations
        fieldTypes.forEach((type) => {
            const el = this.findFirstChild(type);
            // no valid child elements for the type
            if (!el || !(el instanceof SecurePaymentElement) || el.type !== type) {
                return;
            }
            this.elements[type] = el;
            fields[type] = Object.assign(el.getAttributes(), {
                target: this.nodeAsTarget(el),
            });
            if (el.hasStyles()) {
                styles = Object.assign(styles, el.getStyles());
            }
        });
        return { fields, styles };
    }
    /**
     * Grabs the first form child of a given type.
     *
     * To simplify things, integrators implement custom field elements
     * (e.g. `secure-card-number-field`) as children of `secure-payment-form`.
     * This removes the need for custom `id` or `class` attribute values, but
     * the form needs to ensure that only one element of each type is used
     * when configuring the Global Payment JavaScript library.
     *
     * @param type
     *
     * @returns `Element` when a match is found; `null` when no matches
     */
    findFirstChild(type) {
        if (!fieldTypeTagNameMap[type]) {
            return null;
        }
        const elements = this.getElementsByTagName(fieldTypeTagNameMap[type]);
        if (elements.length === 0) {
            return null;
        }
        return elements[0];
    }
    /**
     * Gets the `node`"s `nodeName` property in lower case to be used as the
     * hosted fields target / query selector.
     *
     * @param node
     *
     * @returns Name as a string; Empty string if node is `null`/`undefined`
     */
    nodeAsTarget(node) {
        const name = (node || {}).nodeName;
        return (name || "").toLowerCase();
    }
}
customElements.define("secure-payment-form", SecurePaymentForm);

export { SecureCardCvvField, SecureCardExpirationField, SecureCardNumberField, SecurePaymentElement, SecurePaymentForm, SecureSubmitButton };
//# sourceMappingURL=globalpayments-js-web-components.js.map
